@@ -14,8 +14,6 @@
 #
 
 import logging
-import os
-import yaml
 
 from osc_lib.i18n import _
 
@@ -36,11 +34,6 @@ class UpdatePrepare(DeployOvercloud):
 
     log = logging.getLogger(__name__ + ".MinorUpdatePrepare")
 
-    # enable preservation of all important files (plan env, user env,
-    # roles/network data, user files) so that we don't have to pass
-    # all env files on update command
-    _keep_env_on_update = True
-
     def get_parser(self, prog_name):
         parser = super(UpdatePrepare, self).get_parser(prog_name)
         parser.add_argument('--container-registry-file',
@@ -66,20 +59,9 @@ class UpdatePrepare(DeployOvercloud):
                                    parsed_args.stack)
 
         stack_name = stack.stack_name
-        container_registry = parsed_args.container_registry_file
+        registry = oooutils.load_container_registry(
+            self.log, parsed_args.container_registry_file)
 
-        # Update the container registry:
-        if container_registry:
-            with open(os.path.abspath(container_registry)) as content:
-                registry = yaml.load(content.read())
-        else:
-            self.log.warning(
-                "You have not provided a container registry file. Note "
-                "that none of the containers on your environement will be "
-                "updated. If you want to update your container you have "
-                "to re-run this command and provide the registry file "
-                "with: --container-registry-file option.")
-            registry = None
         # Run update
         ceph_ansible_playbook = parsed_args.ceph_ansible_playbook
         # Run Overcloud deploy (stack update)
@@ -87,6 +69,14 @@ class UpdatePrepare(DeployOvercloud):
         # update_plan_only. The heat stack update is done by the
         # packag_update mistral action
         parsed_args.update_plan_only = True
+
+        # Add the update-prepare.yaml environment to set noops etc
+        templates_dir = (parsed_args.templates or
+                         constants.TRIPLEO_HEAT_TEMPLATES)
+        parsed_args.environment_files = oooutils.prepend_environment(
+            parsed_args.environment_files, templates_dir,
+            constants.UPDATE_PREPARE_ENV)
+
         super(UpdatePrepare, self).take_action(parsed_args)
         package_update.update(clients, container=stack_name,
                               container_registry=registry,
@@ -132,6 +122,13 @@ class UpdateRun(command.Command):
                                    "that all services are updated and running "
                                    "with the target version configuration.")
                             )
+        parser.add_argument("--ssh-user",
+                            dest="ssh_user",
+                            action="store",
+                            default="heat-admin",
+                            help=_("The ssh user name for connecting to "
+                                   "the overcloud nodes.")
+                            )
         parser.add_argument('--static-inventory',
                             dest='static_inventory',
                             action="store",
@@ -154,8 +151,42 @@ class UpdateRun(command.Command):
             nodes = None
         playbook = parsed_args.playbook
         inventory = oooutils.get_tripleo_ansible_inventory(
-            parsed_args.static_inventory)
+            parsed_args.static_inventory, parsed_args.ssh_user)
         oooutils.run_update_ansible_action(self.log, clients, nodes, inventory,
                                            playbook, constants.UPDATE_QUEUE,
                                            constants.MINOR_UPDATE_PLAYBOOKS,
-                                           package_update)
+                                           package_update,
+                                           parsed_args.ssh_user)
+
+
+class UpdateConverge(DeployOvercloud):
+    """Converge the update on Overcloud nodes.
+
+    This restores the plan environment so that normal deployment
+    workflow is back in place. The action does not perform a Heat
+    stack update.
+    """
+
+    log = logging.getLogger(__name__ + ".UpdateConverge")
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)" % parsed_args)
+        clients = self.app.client_manager
+        stack_name = oooutils.get_stack(
+            clients.orchestration, parsed_args.stack).stack_name
+
+        # Only update plan, do not perform stack update.
+        parsed_args.update_plan_only = True
+
+        # Add the update-converge.yaml environment to unset noops
+        templates_dir = (parsed_args.templates or
+                         constants.TRIPLEO_HEAT_TEMPLATES)
+        parsed_args.environment_files = oooutils.prepend_environment(
+            parsed_args.environment_files, templates_dir,
+            constants.UPDATE_CONVERGE_ENV)
+
+        super(UpdateConverge, self).take_action(parsed_args)
+        package_update.update_converge_nodes(
+            clients, container=stack_name, queue_name=constants.UPDATE_QUEUE)
+        print("Update converge on stack {0} complete.".format(
+              parsed_args.stack))
